/**************************************************************************
*
*                            MODULE IR.C
*
***************************************************************************
*           (c) 1998 GKDesign. All Rights Reserved.
***************************************************************************
*
*     Description: Header for Infrared remote control recorder
*
*     Author:         George Katz
*     Date Created:   9/6/97
*
*     Modified by: GK 13/6/98 - Added moving grid lines & variable delay
*
***************************************************************************/
//   The pinout set for the parallel port is as follows:
//
//   Name				Pin			Function
//   ---------------------------------------
//   Data0			2			Data Out 	Active High
//   Acknowledge	10			Data In     Active High
//   Ground		    18-25       circuit common


#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <conio.h>
#include <graphics.h>
#include "ir.h"

unsigned char button1[RECORD_SIZE];  // array to store the data
int Delay=DELAY_DEFAULT;                // changes the timebase on the waveform
int offset = 3;                         // Grid offset from left edge
/*************************************************************************
** Function    : initgraphics
** Description : set up the graphics display
** Parameters  : None
** Return      : None
*************************************************************************/
void initgraphics(void)
{
	union REGS regs;
	int graphdriver;
	int graphmode;
	int graph_error;
	graphdriver = DETECT;
	initgraph(&graphdriver,&graphmode,"");
	graph_error = graphresult();
	if(graph_error < 0)
	{
		puts(grapherrormsg(graph_error));
		exit(1);
	}
}

/*************************************************************************
** Function    : RecordButton
** Description : records the stream from the receiver
** Parameters  : None
** Return      : None
*************************************************************************/
void RecordButton(void)
{
    unsigned int i,j,index;
    unsigned char data,status;
    int x=0,y=10;

    index = 0;

    while(!(inportb(STATUS) & IN_MASK)); // wait for rising edge

    for(i=0;i<RECORD_SIZE;i++)
    {
        data = inportb(STATUS) & IN_MASK;
        if (data)
        {
           button1[index] = data;
           putpixel(x,y-8,RED);
        }
        else
        {
           button1[index] = data;
           putpixel(x,y,RED);
        }
        index++;
        x++;
        if(x>638)
        {
            x = 0;
            y += 15;
            if (y>400)
               break;
        }
        for(j=0;j<Delay;j++) ;   // wait a bit
    }
}

/*************************************************************************
** Function    : PlayButton
** Description : replays the buffered data
** Parameters  : none
** Return      : none
*************************************************************************/
void PlayButton(void)
{
    unsigned int i,j,index;
    unsigned char data,status;
    int x=0,y=10;

    index = 0;

    for(i=0;i<RECORD_SIZE;i++)
    {
        data = button1[index];
        if (data)
        {
           outportb(OUTPUT,0xff);
           putpixel(x,y-8,YELLOW);
        }
        else
        {
           outportb(OUTPUT,0x00);
           putpixel(x,y,YELLOW);
        }
        index++;
        x++;
        if(x>638)
        {
            x = 0;
            y += 15;
            if (y>400)
               break;
        }
        for(j=0;j<Delay;j++) ;
    }
    outportb(OUTPUT,0x00);
}

void DrawBuffer(void)
{
    unsigned int i,j,index;
    unsigned char data;
    int x=0,y=10;

    index = 0;

    for(i=0;i<RECORD_SIZE;i++)
    {
        data = button1[index];
        if (data)
           putpixel(x,y-8,RED);
        else
           putpixel(x,y,RED);
        index++;
        x++;
        if(x>638)
        {
            x = 0;
            y += 15;
            if (y>400)
               break;
        }
    }
}

/*************************************************************************
** Function    : DrawGrid
** Description : Draws a grin on screen to compare the waveform against
** Parameters  : None
** Return      : None
*************************************************************************/
#define STEP 10
void DrawGrid(void)
{
    int i;

    cleardevice();
    if (offset > STEP)
       offset = STEP;
    if (offset <= 0)
       offset = 0;

    setcolor(BLUE);
    for(i=offset;i<639;i+=STEP)
       line(i,0,i,359);

    setcolor(CYAN);
    for(i=offset;i<639;i+=STEP * 5)
       line(i,0,i,359);

}

/*************************************************************************
** Function    : Main program entry point
** Description : main
** Parameters  : None
** Return      : None
*************************************************************************/
main()
{
    char ch='a';

    clrscr();
    printf("\n\n\n\n\n\n\n\n");
    printf("-------------------------------------------------------------------------------\n");
    printf("                       Infrared Remote Control Recorder  V1.1 \n");
    printf("                             Copyright (c) 1998 GKDesign\n");
    printf("-------------------------------------------------------------------------------\n");
    printf("    Please read the README file for instructions on how to use this program\n");
    printf("-------------------------------------------------------------------------------\n");
    printf("\n\n\n\n                              Press any key to begin");

    getch();
    initgraphics();

    DrawGrid();
    while (ch != 'x')   // main loop
    {
        gotoxy(1,25);
        printf("[R] Record, [P] Play, [+/-] Move Grid, [1/2] Change Delay:%d [X] Exit ",Delay);
        ch = getch();
        if(ch == 'r')
        {
            cleardevice();
            DrawGrid();
            gotoxy(1,25);
            printf("Point the Remote at the receiver and press a button.");
            RecordButton();
        }

        if(ch == 'p')
        {
            cleardevice();
            DrawGrid();
            PlayButton();
        }

        if(ch == '+')
        {
            offset++;
            DrawGrid();
            DrawBuffer();
        }

        if(ch == '-')
        {
            offset--;
            DrawGrid();
            DrawBuffer();
        }
        if(ch == '1')
        {
            Delay-=100;
        }

        if(ch == '2')
        {
            Delay+=100;
        }
    }
    closegraph();
    return(0);
}

